@@ -4,17 +4,18 @@
 #' @import data.table jsonlite
 #' @param src source object, result of call to src
 #' @param key (character) A key (collection for mongo)
-#' @param limit (integer) number of records/rows to return. by default
-#' not passed, so you get all results. Only works for CouchDB, 
-#' Elasticsearch and MongoDB; ignored for others
+#' @param limit (integer) number of records/rows to return.
+#' By default not passed, so you get all results.
+#' Only works for CouchDB, Elasticsearch, MongoDB and
+#' RSQLite; ignored for others
 #' @param ... passed on to functions:
-#' 
+#'
 #' - CouchDB: passed to [sofa::db_alldocs()]
 #' - Elasticsearch: passed to [elastic::Search()]
 #' - Redis: ignored
 #' - MongoDB: ignored
 #' - SQLite: ignored
-#' 
+#'
 #' @template deets
 #' @examples \dontrun{
 #' # CouchDB
@@ -44,28 +45,28 @@
 #' docdb_create(src, "mtcars", mtcars)
 #' docdb_get(src, "mtcars")
 #' docdb_get(src, "mtcars", limit = 4)
-#' 
+#'
 #' # SQLite
 #' src <- src_sqlite()
 #' docdb_create(src, "mtcars", mtcars)
-#' docdb_get(src, "mtcars")
 #' docdb_get(src, "mtcars", limit = 4L)
 #' }
-docdb_get <- function(src, key, limit = NULL, ...){
+docdb_get <- function(src, key, limit = NULL, ...) {
   UseMethod("docdb_get")
 }
 
 #' @export
 docdb_get.src_couchdb <- function(src, key, limit = NULL, ...) {
-  assert(key, 'character')
+  assert(key, "character")
   dropmeta(makedf(
-    pluck(sofa::db_alldocs(src$con, dbname = key,
-                           include_docs = TRUE, limit = limit, ...)$rows, "doc")))
+    pluck(sofa::db_alldocs(
+      src$con, dbname = key,
+      include_docs = TRUE, limit = limit, ...)$rows, "doc")))
 }
 
 #' @export
-docdb_get.src_elastic <- function(src, key, limit = NULL, ...){
-  assert(key, 'character')
+docdb_get.src_elastic <- function(src, key, limit = NULL, ...) {
+  assert(key, "character")
   ids <- pluck(elastic::Search(src$con, key, source = FALSE,
                                size = limit, ...)$hits$hits, "_id", "")
   tmp <- elastic::docs_mget(src$con, index = key, type = key, ids = ids,
@@ -75,7 +76,7 @@ docdb_get.src_elastic <- function(src, key, limit = NULL, ...){
 
 #' @export
 docdb_get.src_redis <- function(src, key, limit = NULL, ...) {
-  assert(key, 'character')
+  assert(key, "character")
   res <- src$con$GET(key)
   if (is.null(res)) stop("no matching result found")
   redux::string_to_object(res)
@@ -83,58 +84,55 @@ docdb_get.src_redis <- function(src, key, limit = NULL, ...) {
 
 #' @export
 docdb_get.src_mongo <- function(src, key, limit = NULL, ...) {
-  
+
   # check expectations
-  if (exists("key", inherits = FALSE) && 
-      src$collection != key) 
+  if (exists("key", inherits = FALSE) &&
+      src$collection != key)
     message("Parameter 'key' is different from parameter 'collection', ",
             "was given as ", src$collection, " in src_mongo().")
-  
-  # FIXME: or use $find() here? not if doing a separate query method
-  if (!is.null(limit)) return(src$con$iterate(limit = limit)$page())
-  dump <- tempfile()
-  src$con$export(file(dump))
-  # remove first column, a mongodb identifier
-  jsonlite::stream_in(file(dump), verbose = FALSE) # [,-1]
+
+  # set limit if null
+  if (is.null(limit)) limit <- 0L
+
+  # get data. note: find() does not include _id
+  src$con$find(limit = limit, ...)
+
 }
 
 #' @export
 docdb_get.src_sqlite <- function(src, key, limit = NULL, ...) {
-  
+
   assert(key, "character")
   assert(limit, "integer")
-  
+
   # arguments for call
-  statement <- paste0(
-    # _id is included into json for use with jsonlite::stream_in
-    "SELECT '{\"_id\": \"' || _id || '\", ' || substr(json, 2) ", 
-    "FROM ", key, " ;")
-  
+  statement <- paste0("SELECT json FROM \"", key, "\" ;")
+
   # set limit if not null
   n <- -1L
   if (!is.null(limit)) n <- limit
-  
+
   # temporary file for streaming
   tfname <- tempfile()
   dump <- file(description = tfname,
                encoding = "UTF-8")
-  
+
   # register to remove file
   # after used for streaming
   on.exit(unlink(tfname))
-  
+
   # get data, write to file in ndjson format
   cat(stats::na.omit(unlist(
     DBI::dbGetQuery(conn = src$con,
-                    statement = statement, 
+                    statement = statement,
                     n = n))
   ),
   sep = "\n", # ndjson
   file = dump)
-  
+
   # from jsonlite documentation:
-  # Because parsing huge JSON strings is difficult and inefficient, 
-  # JSON streaming is done using lines of minified JSON records, a.k.a. ndjson. 
+  # Because parsing huge JSON strings is difficult and inefficient,
+  # JSON streaming is done using lines of minified JSON records, a.k.a. ndjson.
   jsonlite::stream_in(dump, verbose = FALSE)
 
 }
